"use strict";

const DataLoader = require("dataloader");
const mapObj = require("map-obj");

let dataSourceFactories = {
	lsblk: require("./data-sources/lsblk"),
	findmnt: require("./data-sources/findmnt"),
	smartctlInfo: require("./data-sources/smartctl/info"),
	smartctlScan: require("./data-sources/smartctl/scan"),
	smartctlAttributes: require("./data-sources/smartctl/attributes"),
	lvmPhysicalVolumes: require("./data-sources/lvm/physical-volumes"),
	lvmVolumeGroups: require("./data-sources/lvm/volume-groups"),
	nvmeListNamespaces: require("./data-sources/nvme/list-namespaces"),
	nvmeIdentifyController: require("./data-sources/nvme/identify-controller"),
};

module.exports = function createLoaders() {
	return mapObj(dataSourceFactories, (name, factory) => {
		return [
			name,
			new DataLoader(factory())
		];
	});
};
